#include "stdafx.h"
#include "xmlutils.h"


// Helper function to create a VT_BSTR variant from a null terminated string. 
HRESULT VariantFromString(PCWSTR wszValue, VARIANT &Variant)
{
	HRESULT hr = S_OK;
	BSTR bstr = SysAllocString(wszValue);
	CHK_ALLOC(bstr);

	V_VT(&Variant) = VT_BSTR;
	V_BSTR(&Variant) = bstr;

CleanUp:
	return hr;
}

// Helper function to create a DOM instance. 
HRESULT CreateAndInitDOM(IXMLDOMDocument **ppDoc)
{
	HRESULT hr = CoCreateInstance(CLSID_DOMDocument, NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(ppDoc));
	if (SUCCEEDED(hr))
	{
		// these methods should not fail so don't inspect result
		(*ppDoc)->put_async(VARIANT_FALSE);
		(*ppDoc)->put_validateOnParse(VARIANT_FALSE);
		(*ppDoc)->put_resolveExternals(VARIANT_FALSE);
		(*ppDoc)->put_preserveWhiteSpace(VARIANT_TRUE);
	}
	return hr;
}

// Helper that allocates the BSTR param for the caller.
HRESULT CreateElement(IXMLDOMDocument *pXMLDom, PCWSTR wszName, IXMLDOMElement **ppElement)
{
	HRESULT hr = S_OK;
	*ppElement = NULL;

	BSTR bstrName = SysAllocString(wszName);
	CHK_ALLOC(bstrName);
	CHK_HR(pXMLDom->createElement(bstrName, ppElement));

CleanUp:
	SysFreeString(bstrName);
	return hr;
}

// Helper function to append a child to a parent node.
HRESULT AppendChildToParent(IXMLDOMNode *pChild, IXMLDOMNode *pParent)
{
	HRESULT hr = S_OK;
	IXMLDOMNode *pChildOut = NULL;
	CHK_HR(pParent->appendChild(pChild, &pChildOut));

CleanUp:
	SAFE_RELEASE(pChildOut);
	return hr;
}

// Helper function to create and add a processing instruction to a document node.
HRESULT CreateAndAddPINode(IXMLDOMDocument *pDoc, PCWSTR wszTarget, PCWSTR wszData)
{
	HRESULT hr = S_OK;
	IXMLDOMProcessingInstruction *pPI = NULL;

	BSTR bstrTarget = SysAllocString(wszTarget);
	BSTR bstrData = SysAllocString(wszData);
	CHK_ALLOC(bstrTarget && bstrData);

	CHK_HR(pDoc->createProcessingInstruction(bstrTarget, bstrData, &pPI));
	CHK_HR(AppendChildToParent(pPI, pDoc));

CleanUp:
	SAFE_RELEASE(pPI);
	SysFreeString(bstrTarget);
	SysFreeString(bstrData);
	return hr;
}

// Helper function to create and add a comment to a document node.
HRESULT CreateAndAddCommentNode(IXMLDOMDocument *pDoc, PCWSTR wszComment)
{
	HRESULT hr = S_OK;
	IXMLDOMComment *pComment = NULL;

	BSTR bstrComment = SysAllocString(wszComment);
	CHK_ALLOC(bstrComment);

	CHK_HR(pDoc->createComment(bstrComment, &pComment));
	CHK_HR(AppendChildToParent(pComment, pDoc));

CleanUp:
	SAFE_RELEASE(pComment);
	SysFreeString(bstrComment);
	return hr;
}

// Helper function to create and add an attribute to a parent node.
HRESULT CreateAndAddAttributeNode(IXMLDOMDocument *pDoc, PCWSTR wszName, PCWSTR wszValue, IXMLDOMElement *pParent)
{
	HRESULT hr = S_OK;
	IXMLDOMAttribute *pAttribute = NULL;
	IXMLDOMAttribute *pAttributeOut = NULL; // Out param that is not used

	BSTR bstrName = NULL;
	VARIANT varValue;
	VariantInit(&varValue);

	bstrName = SysAllocString(wszName);
	CHK_ALLOC(bstrName);
	CHK_HR(VariantFromString(wszValue, varValue));

	CHK_HR(pDoc->createAttribute(bstrName, &pAttribute));
	CHK_HR(pAttribute->put_value(varValue));
	CHK_HR(pParent->setAttributeNode(pAttribute, &pAttributeOut));

CleanUp:
	SAFE_RELEASE(pAttribute);
	SAFE_RELEASE(pAttributeOut);
	SysFreeString(bstrName);
	VariantClear(&varValue);
	return hr;
}

// Helper function to create and append a text node to a parent node.
HRESULT CreateAndAddTextNode(IXMLDOMDocument *pDoc, PCWSTR wszText, IXMLDOMNode *pParent)
{
	HRESULT hr = S_OK;
	IXMLDOMText *pText = NULL;

	BSTR bstrText = SysAllocString(wszText);
	CHK_ALLOC(bstrText);

	CHK_HR(pDoc->createTextNode(bstrText, &pText));
	CHK_HR(AppendChildToParent(pText, pParent));

CleanUp:
	SAFE_RELEASE(pText);
	SysFreeString(bstrText);
	return hr;
}

// Helper function to create and append a CDATA node to a parent node.
HRESULT CreateAndAddCDATANode(IXMLDOMDocument *pDoc, PCWSTR wszCDATA, IXMLDOMNode *pParent)
{
	HRESULT hr = S_OK;
	IXMLDOMCDATASection *pCDATA = NULL;

	BSTR bstrCDATA = SysAllocString(wszCDATA);
	CHK_ALLOC(bstrCDATA);

	CHK_HR(pDoc->createCDATASection(bstrCDATA, &pCDATA));
	CHK_HR(AppendChildToParent(pCDATA, pParent));

CleanUp:
	SAFE_RELEASE(pCDATA);
	SysFreeString(bstrCDATA);
	return hr;
}

// Helper function to create and append an element node to a parent node, and pass the newly created
// element node to caller if it wants.
HRESULT CreateAndAddElementNode(IXMLDOMDocument *pDoc, PCWSTR wszName, PCWSTR wszNewline, IXMLDOMNode *pParent, IXMLDOMElement **ppElement)
{
	HRESULT hr = S_OK;
	IXMLDOMElement* pElement = NULL;

	CHK_HR(CreateElement(pDoc, wszName, &pElement));
	// Add NEWLINE+TAB for identation before this element.
	CHK_HR(CreateAndAddTextNode(pDoc, wszNewline, pParent));
	// Append this element to parent.
	CHK_HR(AppendChildToParent(pElement, pParent));

CleanUp:
	if (ppElement)
		*ppElement = pElement;  // Caller is repsonsible to release this element.
	else
		SAFE_RELEASE(pElement); // Caller is not interested on this element, so release it.

	return hr;
}


HRESULT SelectSingleNode(IXMLDOMDocument *pDoc, PCWSTR pszExpr, IXMLDOMNode **ppResultNode)
{
	HRESULT hr = S_OK;
	IXMLDOMText *pText = NULL;

	BSTR queryString = SysAllocString(pszExpr);
	CHK_ALLOC(queryString);
	CHK_HR(pDoc->selectSingleNode(queryString, ppResultNode));

CleanUp:
	SAFE_RELEASE(pText);
	SysFreeString(queryString);
	return hr;
}


HRESULT SelectNodes(IXMLDOMDocument *pDoc, PCWSTR pszExpr, IXMLDOMNodeList **ppResultList)
{
	HRESULT hr = S_OK;
	IXMLDOMText *pText = NULL;

	BSTR queryString = SysAllocString(pszExpr);
	CHK_ALLOC(queryString);
	CHK_HR(pDoc->selectNodes(queryString, ppResultList));

CleanUp:
	SAFE_RELEASE(pText);
	SysFreeString(queryString);
	return hr;
}

HRESULT SelectNodes2(IXMLDOMNode *pNode, PCWSTR pszExpr, IXMLDOMNodeList **ppResultList)
{
	HRESULT hr = S_OK;
	IXMLDOMText *pText = NULL;

	BSTR queryString = SysAllocString(pszExpr);
	CHK_ALLOC(queryString);
	CHK_HR(pNode->selectNodes(queryString, ppResultList));

CleanUp:
	SAFE_RELEASE(pText);
	SysFreeString(queryString);
	return hr;
}