﻿

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.Ports;


namespace MTKF31
{
    class CommunicationUtil
    {
        const byte STX_F31 = 0xF2;
        const byte ETX = 0x03;
        const byte ENQ = 0x05;
        const byte ACK = 0x06;
        const byte NAK = 0x15;

        static byte[] _msgbuf = new byte[1024];

        public static int SendF31Command(USARTStream stream, byte address, byte[] text)
        {
            return SendF31Command(stream, address, text, null);
        }


        public static int SendF31Command(USARTStream stream, byte address, byte[] cdt1, byte[] cdt2)
        {
            int msglen = 0;
            int textlen = cdt1.Length + 1;
            if (cdt2 != null)
                textlen += cdt2.Length;

            _msgbuf[msglen++] = STX_F31;
            _msgbuf[msglen++] = address;
            _msgbuf[msglen++] = (byte)((textlen >> 8) & 0xff);
            _msgbuf[msglen++] = (byte)(textlen & 0xff);
            _msgbuf[msglen++] = (byte) 'C';
            Array.Copy(cdt1, 0, _msgbuf, msglen, cdt1.Length);
            msglen += cdt1.Length;

            if (cdt2 != null)
            {
                Array.Copy(cdt2, 0, _msgbuf, msglen, cdt2.Length);
                msglen += cdt2.Length;
            }

            _msgbuf[msglen++] = ETX;
            _msgbuf[msglen] = CalcBCC(_msgbuf, 0, msglen);
            msglen++;

            try
            {
                stream.Write(_msgbuf, 0, msglen);
                return 0;
            }
            catch (TimeoutException)
            {
                return Errors.TIMEOUT;
            }
        }


        public static int SendACK(USARTStream stream)
        {
            try
            {
                byte[] buf = { (byte)ACK };
                stream.Write(buf, 0, 1);
                return 0;
            }
            catch (TimeoutException)
            {
                return Errors.TIMEOUT;
            }
        }

        public static int SendENQ(USARTStream stream)
        {
            try
            {
                byte[] buf = { (byte)ENQ };
                stream.Write(buf, 0, 1);
                return 0;
            }
            catch (TimeoutException)
            {
                return Errors.TIMEOUT;
            }
        }


        public static int WaitForACK(USARTStream stream)
        {
            try
            {
                byte[] buf = new byte[1];

                stream.WaitBytesAvailable(1, 2000);
                stream.Read(buf, 0, 1);

                if (buf[0] != ACK)
                    return Errors.RECEPTION_ERROR;

                return 0;
            }
            catch (TimeoutException)
            {
                return Errors.TIMEOUT;
            }
        }

        public static int WaitForF31Response(USARTStream stream, int timeout, int minRDTLen, ref byte[] rdata)
        {
            try
            {
                stream.WaitBytesAvailable(4, timeout);
                stream.Read(_msgbuf, 0, 4);

                int count = (_msgbuf[2] << 8) + _msgbuf[3] + 2;
                stream.WaitBytesAvailable(count, 2000);
                stream.Read(_msgbuf, 4, count);

                int msglen = count + 4;
                byte bcc = CalcBCC(_msgbuf, 0, msglen - 1);
                if (bcc != _msgbuf[msglen - 1])
                    return Errors.RECEPTION_ERROR;

                switch (_msgbuf[4] & 0xff)
                {
                    case 0x50:
                        {
                            int rlen = msglen - 9;
                            if (rlen < minRDTLen)
                                return Errors.RECEPTION_ERROR;

                            rdata = new byte[rlen];
                            Array.Copy(_msgbuf, 7, rdata, 0, rlen);
                            return 0;
                        }

                    case 0x4E:
                        return (_msgbuf[7] << 8) + _msgbuf[8];
                    default:
                        return Errors.RECEPTION_ERROR;
                }
            }
            catch (TimeoutException)
            {
                return Errors.TIMEOUT;
            }
        }


        public static byte CalcBCC(byte[] buf, int offset, int count)
        {
            byte b = 0;
            for (int i = 0; i < count; i++)
            {
                b ^= buf[i + offset];
            }

            return b;
        }


        public static byte CalcBCC(List<byte> buf, int offset, int count)
        {
            byte b = 0;
            for (int i = 0; i < count; i++)
            {
                b ^= buf[i + offset];
            }

            return b;
        }

    }
}
