﻿using System;
using System.IO.Ports;

namespace MTKF31
{
    class MTKF31DeviceObj
    {
        USARTStream _stream = new USARTStream();
        byte _address = 0;

        public static string err2str(int ret)
        {
            switch (ret)
            {
                case Errors.ACCESS_DENIED: return "Access denied";
                case Errors.TIMEOUT: return "Communication Timeout";
                case Errors.RECEPTION_ERROR: return "Reception error";
                case Errors.INTERNAL_ERROR: return "Internal error";

                default:
                    if (ret > 0)
                    {
                        byte[] codes = new byte[2];
                        codes[0] = (byte)((ret >> 8) & 0xff); // e1
                        codes[1] = (byte)(ret & 0xff); // e0
                        return String.Format("FW error: {0}", System.Text.Encoding.Default.GetString(codes));
                    }
                    return "Other error";
            }
        }

        public int Connect(string portName, byte address)
        {
            try
            {
                _stream.Open(portName, 9600);
                _address = address;
            }
            catch (System.UnauthorizedAccessException)
            {
                return Errors.ACCESS_DENIED;
            }
            catch (Exception)
            {
                return Errors.INTERNAL_ERROR;
            }

            return 0;
        }


        public int Disconnect()
        {
            if (_stream.IsOpen)
            {
                _stream.Close();
            }

            return 0;
        }


        public bool IsConnected
        {
            get { return _stream.IsOpen; }
        }

        public int Reset(MTKF31ResetAction action, ref string fwversion)
        {
            byte[] text;
            switch (action)
            {
                case MTKF31ResetAction.Eject:
                    text = new byte[] { 0x30, 0x30 };
                    break;

                case MTKF31ResetAction.Retain:
                    text = new byte[] { 0x30, 0x31 };
                    break;

                case MTKF31ResetAction.NoAction:
                default:
                    text = new byte[] { 0x30, 0x33 };
                    break;
            }

            byte[] rdata = null;
            int ret = Execute(text, 3, ref rdata, 10000);
            if (ret != 0)
                return ret;

            int fwvlen = rdata.Length - 3; // -3: st0 st1 st2
            if (fwvlen > 0)
                fwversion = System.Text.Encoding.Default.GetString(rdata, 3, fwvlen);

            return 0;
        }


        public int GetStatus(ref MTKF31Status status)
        {
            byte[] text = new byte[] { 0x31, 0x30 };
            byte[] rdata = null;
            int ret = Execute(text, 3, ref rdata, 5000);
            if (ret != 0)
                return ret;

            switch (rdata[0])
            {
                case 0x30:
                    status = MTKF31Status.NoCardPresent;
                    break;
                case 0x31:
                    status = MTKF31Status.CardAtExitSlot;
                    break;
                case 0x32:
                    status = MTKF31Status.CardAtReadPos;
                    break;
                default:
                    return Errors.RECEPTION_ERROR;
            }

            return 0;
        }

        public int AllowInsertion(bool isAllowed)
        {
            byte[] text1 = new byte[] { 0x33, 0x30 }; //Enable Insertion
            byte[] text2 = new byte[] { 0x33, 0x31 }; //Forbid Insertion
            byte[] rdata = null;
            if(isAllowed)
            return Execute(text1, 3, ref rdata, 5000);
            else
            return Execute(text2, 3, ref rdata, 5000);

        }
        public int Dispense()
        {
            byte[] text = new byte[] { 0x32, 0x30 };
            byte[] rdata = null;
            return Execute(text, 3, ref rdata, 5000);
        }

        public int Eject()
        {
            byte[] text = new byte[] { 0x32, 0x39 };
            byte[] rdata = null;
            return Execute(text, 3, ref rdata, 5000);
        }


        public int Retain()
        {
            byte[] text = new byte[] { 0x32, 0x33 };
            byte[] rdata = null;
            return Execute(text, 3, ref rdata, 5000);
        }


        public int MoveCardToICCPos()
        {
            byte[] text = new byte[] { 0x32, 0x31 };
            byte[] rdata = null;
            return Execute(text, 3, ref rdata, 5000);
        }

        public int MoveCardToRFPos()
        {
            byte[] text = new byte[] { 0x32, 0x32 };
            byte[] rdata = null;
            return Execute(text, 3, ref rdata, 5000);
        }

        public int ICCPowerOn(ref bool isT0, ref byte[] atr)
        {
            byte[] text = new byte[] { 0x51, 0x30, 0x30 }; // EMV
            byte[] rdata = null;
            int ret = Execute(text, 4, ref rdata, 5000);
            if (ret != 0)
                return ret;

            isT0 = rdata[3] == 0x30 ? true : false;

            int atrlen = rdata.Length - 4;
            if (atrlen > 0)
            {
                atr = new byte[atrlen];
                Array.Copy(rdata, 4, atr, 0, atrlen);
            }

            return 0;
        }


        public int ICCPowerOff()
        {
            byte[] text = new byte[] { 0x51, 0x31, };
            byte[] rdata = null;
            return Execute(text, 3, ref rdata, 5000);
        }

        public int RFActivate(ref bool isTypeA, ref byte[] atr)
        {
            byte[] text = new byte[] { 0x60, 0x30, 0x41,0x42 }; // Activate Sequence; 0x410x42 Type A->B,0x42 0x41 Type B->A,  0x41,0x0 Type A only, 0x42,0x0 TypeB only
            byte[] rdata = null;
            int ret = Execute(text, 4, ref rdata, 5000);
            if (ret != 0)
                return ret;

            isTypeA = rdata[3] == 0x41 ? true : false;

            int atrlen = rdata.Length - 4;
            if (atrlen > 0)
            {
                atr = new byte[atrlen];
                Array.Copy(rdata, 4, atr, 0, atrlen);
            }

            return 0;
        }
        public int RFDeactivate()
        {
            byte[] text = new byte[] { 0x60, 0x31,};
            byte[] rdata = null;
            return Execute(text, 3, ref rdata, 5000);
        }

        public int ICCTransmitT0(byte[] sendData, ref byte[] receivedData)
        {
            byte[] cdt1 = new byte[] { 0x51, 0x33, };
            byte[] rdata = null;
            int ret = Execute(cdt1, sendData, 5, ref rdata, 5000);
            if (ret != 0)
                return ret;

            int recvlen = rdata.Length - 3;
            receivedData = new byte[recvlen];
            Array.Copy(rdata, 3, receivedData, 0, recvlen);

            return 0;
        }

        public int ICCTransmitT1(byte[] sendData, ref byte[] receivedData)
        {
            byte[] cdt1 = new byte[] { 0x51, 0x34 };
            byte[] rdata = null;
            int ret = Execute(cdt1, sendData, 5, ref rdata, 5000);
            if (ret != 0)
                return ret;

            int recvlen = rdata.Length - 3;
            receivedData = new byte[recvlen];
            Array.Copy(rdata, 3, receivedData, 0, recvlen);

            return 0;
        }
        public int RFTransmitTypeA(byte[] sendData, ref byte[] receivedData)
        {
            byte[] cdt1 = new byte[] { 0x60, 0x34 };  //For Type B 0x60 0x35
            byte[] rdata = null;
            int ret = Execute(cdt1, sendData, 5, ref rdata, 5000);
            if (ret != 0)
                return ret;

            int recvlen = rdata.Length - 3;
            receivedData = new byte[recvlen];
            Array.Copy(rdata, 3, receivedData, 0, recvlen);

            return 0;
        }


        private int Execute(byte[] text, int minRDTLen, ref byte[] rdata, int timeout)
        {
            return Execute(text, null, minRDTLen, ref rdata, timeout);
        }


        private int Execute(byte[] cdt1, byte[] cdt2, int minRDTLen, ref byte[] rdata, int timeout)
        {
            int ret;

            _stream.DiscardIOBuffers();

            ret = CommunicationUtil.SendF31Command(_stream, _address, cdt1, cdt2);
            if (ret != 0)
                return ret;

            ret = CommunicationUtil.WaitForACK(_stream);
            if (ret != 0)
                return ret;

            ret = CommunicationUtil.WaitForF31Response(_stream, timeout, minRDTLen, ref rdata);
            if (ret != 0)
                return ret;

            CommunicationUtil.SendACK(_stream);

            return 0;
        }
    }
}
